"""AdCP Type System.

All AdCP types exported from a single location.
Users should import from here or directly from adcp.

    from adcp.types import Product, CreativeFilters
    from adcp import Product, CreativeFilters

IMPORTANT: Never import directly from adcp.types.generated_poc or
adcp.types._generated. These are internal modules regenerated from
upstream schemas. Only import from adcp.types (this module) or adcp.

Type Coercion:
    Request types accept flexible input for developer ergonomics:

    - Enum fields accept string values:
        ListCreativeFormatsRequest(type="video")  # Works!
        ListCreativeFormatsRequest(type=FormatCategory.video)  # Also works

    - Context fields accept dicts:
        GetProductsRequest(context={"key": "value"})  # Works!

    - FieldModel lists accept strings:
        ListCreativesRequest(fields=["creative_id", "name"])  # Works!

    See adcp.types._ergonomic for implementation details.
"""

from __future__ import annotations

# Apply type coercion to generated types (must be imported before other types)
from adcp.types import (
    _ergonomic,  # noqa: F401
    aliases,  # noqa: F401
)

# Also make submodules available for advanced use
from adcp.types import _generated as generated  # noqa: F401

# Import all types from generated code
# V3 Protocol Discovery types
# V3 Content Standards types
# V3 Sponsored Intelligence types
# V3 Governance (Property Lists) types
# Re-export webhook payload type for webhook handling
from adcp.types._generated import (
    A2UiComponent,
    A2UiSurface,
    Account,
    AccountReference,
    AccountScope,
    ActivateSignalRequest,
    ActivateSignalResponse,
    AggregatedTotals,
    Artifact,
    ArtifactWebhookPayload,
    Asset,
    AssetContentType,
    AssetSelectors,
    AssignedPackage,
    Assignments,
    AudienceSource,
    AudioAsset,
    Authentication,
    AuthenticationScheme,
    AuthorizedAgents,
    AvailableMetric,
    BrandManifest,
    BrandReference,
    BuildCreativeRequest,
    BuildCreativeResponse,
    BuyingMode,
    ByCatalogItemItem,
    ByPackageItem,
    CalibrateContentRequest,
    CalibrateContentResponse,
    Catalog,
    CatalogAction,
    CatalogFieldBinding,
    CatalogFieldBinding1,
    CatalogFieldMapping,
    CatalogItemStatus,
    CatalogRequirements,
    CatalogType,
    CoBrandingRequirement,
    Colors,
    Contact,
    ContentIdType,
    ContentStandards,
    ContextObject,
    Country,
    CpaPricingOption,
    CpcPricingOption,
    CpcvPricingOption,
    CpmPricingOption,
    CppPricingOption,
    CpvPricingOption,
    CreateContentStandardsRequest,
    CreateContentStandardsResponse,
    CreateMediaBuyRequest,
    CreateMediaBuyResponse,
    CreatePropertyListRequest,
    CreatePropertyListResponse,
    Creative,
    CreativeAction,
    CreativeAgent,
    CreativeAgentCapability,
    CreativeApproval,
    CreativeApprovalStatus,
    CreativeAsset,
    CreativeAssignment,
    CreativeFilters,
    CreativeManifest,
    CreativePolicy,
    CreativeStatus,
    CreativeVariant,
    CssAsset,
    DaastTrackingEvent,
    DaastVersion,
    DailyBreakdownItem,
    DateRange,
    DatetimeRange,
    DayOfWeek,
    DaypartTarget,
    DeletePropertyListRequest,
    DeletePropertyListResponse,
    DeliverTo,
    DeliveryForecast,
    DeliveryMeasurement,
    DeliveryMetrics,
    DeliveryStatus,
    DeliveryType,
    DemographicSystem,
    DevicePlatform,
    DeviceType,
    DimensionUnit,
    Disclaimer,
    DoohMetrics,
    Duration,
    Error,
    ErrorCode,
    EventType,
    ExtensionObject,
    FeedbackSource,
    FeedFormat,
    Field1,
    FieldModel,
    FlatRatePricingOption,
    Fonts,
    ForecastableMetric,
    ForecastMethod,
    ForecastPoint,
    ForecastRange,
    ForecastRangeUnit,
    Format,
    FormatCard,
    FormatCardDetailed,
    FormatCategory,
    FormatId,
    FormatIdParameter,
    FrequencyCap,
    FrequencyCapScope,
    GeoCountry,
    GeoMetro,
    GeoPostalArea,
    GeoRegion,
    GetAccountFinancialsRequest,
    GetAccountFinancialsResponse,
    GetAdcpCapabilitiesRequest,
    GetAdcpCapabilitiesResponse,
    GetContentStandardsRequest,
    GetContentStandardsResponse,
    GetCreativeDeliveryRequest,
    GetCreativeDeliveryResponse,
    GetCreativeFeaturesRequest,
    GetCreativeFeaturesResponse,
    GetMediaBuyArtifactsRequest,
    GetMediaBuyArtifactsResponse,
    GetMediaBuyDeliveryRequest,
    GetMediaBuyDeliveryResponse,
    GetMediaBuysRequest,
    GetMediaBuysResponse,
    GetProductsRequest,
    GetProductsResponse,
    GetPropertyListRequest,
    GetPropertyListResponse,
    GetSignalsRequest,
    GetSignalsResponse,
    Gtin,
    HtmlAsset,
    HttpMethod,
    Identifier,
    ImageAsset,
    Input,
    JavascriptAsset,
    JavascriptModuleType,
    LandingPageRequirement,
    ListAccountsRequest,
    ListAccountsResponse,
    ListAuthorizedPropertiesRequest,
    ListAuthorizedPropertiesResponse,
    ListContentStandardsRequest,
    ListContentStandardsResponse,
    ListCreativeFormatsRequest,
    ListCreativeFormatsResponse,
    ListCreativesRequest,
    ListCreativesResponse,
    ListPropertyListsRequest,
    ListPropertyListsResponse,
    LogEventRequest,
    LogEventResponse,
    Logo,
    MarkdownFlavor,
    McpWebhookPayload,
    MeasurementPeriod,
    MediaBuy,
    MediaBuyDelivery,
    MediaBuyPackage,
    MediaBuyStatus,
    MediaChannel,
    Metadata,
    MetricType,
    NotificationType,
    Offering,
    OfferingAssetConstraint,
    OfferingAssetGroup,
    OptimizationGoal,
    OutcomeMeasurement,
    Overlay,
    Pacing,
    PackageRequest,
    PackageStatus,
    PackageUpdate,
    Pagination,
    PaginationRequest,
    Parameters,
    Performance,
    PerformanceFeedback,
    Placement,
    Preview,
    PreviewCreativeRequest,
    PreviewCreativeResponse,
    PreviewOutputFormat,
    PreviewRender,
    PriceGuidance,
    Pricing,
    PricingModel,
    PrimaryCountry,
    Product,
    ProductCard,
    ProductCardDetailed,
    ProductCatalog,
    ProductFilters,
    PromotedOfferings,
    PromotedOfferingsAssetRequirements,
    PromotedOfferingsRequirement,
    PromotedProducts,
    Property,
    PropertyIdentifierTypes,
    PropertyList,
    PropertyListChangedWebhook,
    PropertyListFilters,
    PropertyListReference,
    PropertyType,
    Proposal,
    ProtocolEnvelope,
    ProtocolResponse,
    ProvidePerformanceFeedbackRequest,
    ProvidePerformanceFeedbackResponse,
    PublisherDomain,
    PublisherIdentifierTypes,
    PushNotificationConfig,
    QuartileData,
    QuerySummary,
    ReachUnit,
    Refine,
    ReportingCapabilities,
    ReportingFrequency,
    ReportingPeriod,
    ReportingWebhook,
    ReportUsageRequest,
    ReportUsageResponse,
    Request,
    Response,
    Responsive,
    Results,
    Security,
    SiCapabilities,
    SiGetOfferingRequest,
    SiGetOfferingResponse,
    Signal,
    SignalCatalogType,
    SignalFilters,
    SignalPricing,
    SignalPricingOption,
    SiIdentity,
    SiInitiateSessionRequest,
    SiInitiateSessionResponse,
    SiSendMessageRequest,
    SiSendMessageResponse,
    SiTerminateSessionRequest,
    SiTerminateSessionResponse,
    SiUiElement,
    Snapshot,
    SnapshotUnavailableReason,
    Sort,
    SortApplied,
    SortDirection,
    StatusSummary,
    SyncAccountsRequest,
    SyncAccountsResponse,
    SyncAudiencesRequest,
    SyncAudiencesResponse,
    SyncCatalogsInputRequired,
    SyncCatalogsRequest,
    SyncCatalogsResponse,
    SyncCatalogsSubmitted,
    SyncCatalogsWorking,
    SyncCreativesRequest,
    SyncCreativesResponse,
    SyncEventSourcesRequest,
    SyncEventSourcesResponse,
    Tags,
    TargetingOverlay,
    TaskType,
    TextAsset,
    TimeBasedPricingOption,
    TimeUnit,
    Totals,
    Transform,
    UpdateContentStandardsRequest,
    UpdateContentStandardsResponse,
    UpdateFrequency,
    UpdateMediaBuyRequest,
    UpdateMediaBuyResponse,
    UpdatePropertyListRequest,
    UpdatePropertyListResponse,
    UrlAsset,
    UrlAssetType,
    ValidateContentDeliveryRequest,
    ValidateContentDeliveryResponse,
    ValidationMode,
    VastTrackingEvent,
    VastVersion,
    VcpmPricingOption,
    VenueBreakdownItem,
    VideoAsset,
    ViewThreshold,
    WcagLevel,
    WebhookAsset,
    WebhookResponseType,
)
from adcp.types._generated import TaskStatus as GeneratedTaskStatus
from adcp.types._generated import _PackageFromPackage as Package

# Import semantic aliases for discriminated unions
from adcp.types.aliases import (
    AccountReferenceById,
    AccountReferenceByNaturalKey,
    ActivateSignalErrorResponse,
    ActivateSignalSuccessResponse,
    AgentDeployment,
    AgentDestination,
    AuthorizedAgent,
    AuthorizedAgentsByInlineProperties,
    AuthorizedAgentsByPropertyId,
    AuthorizedAgentsByPropertyTag,
    AuthorizedAgentsByPublisherProperties,
    AuthorizedAgentsBySignalId,
    AuthorizedAgentsBySignalTag,
    BothPreviewRender,
    BuildCreativeErrorResponse,
    BuildCreativeSuccessResponse,
    CalibrateContentErrorResponse,
    CalibrateContentSuccessResponse,
    CatalogGroupBinding,
    ConsentBasis,
    CpmSignalPricingOption,
    CreateContentStandardsErrorResponse,
    CreateContentStandardsSuccessResponse,
    CreateMediaBuyErrorResponse,
    CreateMediaBuySuccessResponse,
    Deployment,
    Destination,
    FlatFeeSignalPricingOption,
    GetAccountFinancialsErrorResponse,
    GetAccountFinancialsSuccessResponse,
    GetContentStandardsErrorResponse,
    GetContentStandardsSuccessResponse,
    GetCreativeDeliveryByBuyerRefRequest,
    GetCreativeDeliveryByCreativeRequest,
    GetCreativeDeliveryByMediaBuyRequest,
    GetCreativeFeaturesErrorResponse,
    GetCreativeFeaturesSuccessResponse,
    GetMediaBuyArtifactsErrorResponse,
    GetMediaBuyArtifactsSuccessResponse,
    GetProductsBriefRequest,
    GetProductsRefineRequest,
    GetProductsWholesaleRequest,
    GetSignalsDiscoveryRequest,
    GetSignalsLookupRequest,
    HtmlPreviewRender,
    InlineDaastAsset,
    InlineVastAsset,
    KeyValueActivationKey,
    ListContentStandardsErrorResponse,
    ListContentStandardsSuccessResponse,
    LogEventErrorResponse,
    LogEventSuccessResponse,
    MediaBuyDeliveryStatus,
    MediaSubAsset,
    PercentOfMediaSignalPricingOption,
    PlatformDeployment,
    PlatformDestination,
    PreviewCreativeBatchRequest,
    PreviewCreativeBatchResponse,
    PreviewCreativeSingleRequest,
    PreviewCreativeSingleResponse,
    PreviewCreativeVariantRequest,
    PreviewCreativeVariantResponse,
    PricingOption,
    PropertyId,
    PropertyTag,
    ProvidePerformanceFeedbackByBuyerRefRequest,
    ProvidePerformanceFeedbackByMediaBuyRequest,
    ProvidePerformanceFeedbackErrorResponse,
    ProvidePerformanceFeedbackSuccessResponse,
    PublisherProperties,
    PublisherPropertiesAll,
    PublisherPropertiesById,
    PublisherPropertiesByTag,
    SegmentIdActivationKey,
    SiSendActionResponseRequest,
    SiSendTextMessageRequest,
    SyncAccountsErrorResponse,
    SyncAccountsSuccessResponse,
    SyncAudiencesAudience,
    SyncAudiencesErrorResponse,
    SyncAudiencesSuccessResponse,
    SyncCatalogResult,
    SyncCatalogsErrorResponse,
    SyncCatalogsSuccessResponse,
    SyncCreativeResult,
    SyncCreativesErrorResponse,
    SyncCreativesSuccessResponse,
    SyncEventSourcesErrorResponse,
    SyncEventSourcesSuccessResponse,
    TextSubAsset,
    UpdateContentStandardsErrorResponse,
    UpdateContentStandardsSuccessResponse,
    UpdateMediaBuyErrorResponse,
    UpdateMediaBuyPackagesRequest,
    UpdateMediaBuyPropertiesRequest,
    UpdateMediaBuySuccessResponse,
    UrlDaastAsset,
    UrlPreviewRender,
    UrlVastAsset,
    ValidateContentDeliveryErrorResponse,
    ValidateContentDeliverySuccessResponse,
)

# Re-export core types (not in generated, but part of public API)
# Note: We don't import TaskStatus here to avoid shadowing GeneratedTaskStatus
# Users should import TaskStatus from adcp.types.core directly if they need the core enum
from adcp.types.core import (
    AgentConfig,
    Member,
    Protocol,
    ResolvedBrand,
    ResolvedProperty,
    TaskResult,
    WebhookMetadata,
)

# Status: _generated picks invoice status (get_account_financials_response) due to
# alphabetical module sort. Import the delivery status variant directly for backward compat.
from adcp.types.generated_poc.media_buy.get_media_buy_delivery_response import (  # noqa: E501
    Status,
)

# Semantic aliases for auto-generated field enum names
ListCreativesField = Field1

# Backward compatibility aliases
AssetType = AssetContentType  # Use AssetContentType instead
Measurement = OutcomeMeasurement  # Renamed upstream to OutcomeMeasurement

# Pricing type renames: upstream merged auction/fixed variants into single types
CpmAuctionPricingOption = CpmPricingOption
CpmFixedRatePricingOption = CpmPricingOption
VcpmAuctionPricingOption = VcpmPricingOption
VcpmFixedRatePricingOption = VcpmPricingOption

# Activation key schema change: property_id/property_tag → segment_id/key_value
PropertyIdActivationKey = SegmentIdActivationKey
PropertyTagActivationKey = KeyValueActivationKey

# Preview alias renames: format/manifest → single/batch/variant discriminator
PreviewCreativeFormatRequest = PreviewCreativeSingleRequest
PreviewCreativeManifestRequest = PreviewCreativeBatchRequest
PreviewCreativeStaticResponse = PreviewCreativeSingleResponse
PreviewCreativeInteractiveResponse = PreviewCreativeBatchResponse

# Schema renames from filter ref split (v1.0.0)
Action = CreativeAction
Capability = CreativeAgentCapability
CoBranding = CoBrandingRequirement
FormatType = FormatCategory
LandingPage = LandingPageRequirement
Method = HttpMethod
ModuleType = JavascriptModuleType
TrackingEvent = VastTrackingEvent  # Split into DaastTrackingEvent and VastTrackingEvent
Unit = DimensionUnit
OutputFormat = PreviewOutputFormat
ResponseType = WebhookResponseType
Scheme = AuthenticationScheme
SignalType = SignalCatalogType
UrlType = UrlAssetType
AvailableReportingFrequency = ReportingFrequency

__all__ = [
    # A2UI types
    "A2UiComponent",
    "A2UiSurface",
    # Account types
    "Account",
    "AccountReference",
    "AccountReferenceById",
    "AccountReferenceByNaturalKey",
    "AccountScope",
    "GetAccountFinancialsRequest",
    "GetAccountFinancialsResponse",
    "ListAccountsRequest",
    "ListAccountsResponse",
    "ReportUsageRequest",
    "ReportUsageResponse",
    "SyncAccountsRequest",
    "SyncAccountsResponse",
    # Request/Response types
    "ActivateSignalRequest",
    "ActivateSignalResponse",
    "CreativeAction",
    "AggregatedTotals",
    "BuildCreativeRequest",
    "BuildCreativeResponse",
    "ByCatalogItemItem",
    "ByPackageItem",
    "CreateMediaBuyRequest",
    "CreateMediaBuyResponse",
    "DailyBreakdownItem",
    "GetMediaBuyDeliveryRequest",
    "GetMediaBuyDeliveryResponse",
    "GetMediaBuysRequest",
    "GetMediaBuysResponse",
    "GetProductsRequest",
    "GetProductsResponse",
    "GetSignalsRequest",
    "GetSignalsResponse",
    "ListCreativeFormatsRequest",
    "ListCreativeFormatsResponse",
    "ListCreativesRequest",
    "ListCreativesResponse",
    "MediaBuyDelivery",
    "PreviewCreativeRequest",
    "PreviewCreativeResponse",
    "ProtocolEnvelope",
    "ProtocolResponse",
    "ProvidePerformanceFeedbackRequest",
    "ProvidePerformanceFeedbackResponse",
    "QuerySummary",
    "Refine",
    "SortApplied",
    "StatusSummary",
    "SyncCatalogsInputRequired",
    "SyncCatalogsRequest",
    "SyncCatalogsResponse",
    "SyncCatalogsSubmitted",
    "SyncCatalogsWorking",
    "SyncAudiencesAudience",
    "SyncAudiencesRequest",
    "SyncAudiencesResponse",
    "ConsentBasis",
    "SyncCreativesRequest",
    "SyncCreativesResponse",
    # Event & Source Operations
    "LogEventRequest",
    "LogEventResponse",
    "SyncEventSourcesRequest",
    "SyncEventSourcesResponse",
    "Totals",
    "UpdateMediaBuyRequest",
    "UpdateMediaBuyResponse",
    # V3 Protocol Discovery
    "GetAdcpCapabilitiesRequest",
    "GetAdcpCapabilitiesResponse",
    # Creative Delivery
    "GetCreativeDeliveryRequest",
    "GetCreativeDeliveryResponse",
    # Creative Features
    "GetCreativeFeaturesRequest",
    "GetCreativeFeaturesResponse",
    # Forecasting types
    "DayOfWeek",
    "DaypartTarget",
    "DeliveryForecast",
    "DemographicSystem",
    "ForecastMethod",
    "ForecastPoint",
    "ForecastRange",
    "ForecastRangeUnit",
    "ForecastableMetric",
    "ReachUnit",
    # V3 Content Standards
    "Artifact",
    "ArtifactWebhookPayload",
    "CalibrateContentRequest",
    "CalibrateContentResponse",
    "ContentStandards",
    "CreateContentStandardsRequest",
    "CreateContentStandardsResponse",
    "GetContentStandardsRequest",
    "GetContentStandardsResponse",
    "GetMediaBuyArtifactsRequest",
    "GetMediaBuyArtifactsResponse",
    "ListContentStandardsRequest",
    "ListContentStandardsResponse",
    "UpdateContentStandardsRequest",
    "UpdateContentStandardsResponse",
    "ValidateContentDeliveryRequest",
    "ValidateContentDeliveryResponse",
    # V3 Sponsored Intelligence
    "SiCapabilities",
    "SiGetOfferingRequest",
    "SiGetOfferingResponse",
    "SiIdentity",
    "SiInitiateSessionRequest",
    "SiInitiateSessionResponse",
    "SiSendMessageRequest",
    "SiSendMessageResponse",
    "SiTerminateSessionRequest",
    "SiTerminateSessionResponse",
    "SiUiElement",
    # V3 Governance (Property Lists)
    "CreatePropertyListRequest",
    "CreatePropertyListResponse",
    "DeletePropertyListRequest",
    "DeletePropertyListResponse",
    "GetPropertyListRequest",
    "GetPropertyListResponse",
    "ListPropertyListsRequest",
    "ListPropertyListsResponse",
    "PropertyList",
    "PropertyListChangedWebhook",
    "PropertyListFilters",
    "PropertyListReference",
    "UpdatePropertyListRequest",
    "UpdatePropertyListResponse",
    # Catalog types
    "Catalog",
    "CatalogAction",
    "CatalogFieldBinding",
    "CatalogFieldBinding1",
    "CatalogGroupBinding",
    "CatalogFieldMapping",
    "CatalogItemStatus",
    "CatalogRequirements",
    "CatalogType",
    "ContentIdType",
    "Gtin",
    "OfferingAssetConstraint",
    "OfferingAssetGroup",
    # Domain types
    "Asset",
    "AssetSelectors",
    "AssetContentType",
    "AssetType",  # Deprecated
    "AudienceSource",
    "BrandReference",
    "BuyingMode",
    "ContextObject",
    "DateRange",
    "DatetimeRange",
    "Duration",
    "FormatCategory",
    "AssignedPackage",
    "Assignments",
    "BrandManifest",
    "CreativeAgentCapability",
    "CoBrandingRequirement",
    "Colors",
    "Contact",
    "Creative",
    "CreativeAgent",
    "CreativeApproval",
    "CreativeApprovalStatus",
    "CreativeAsset",
    "CreativeAssignment",
    "CreativeFilters",
    "CreativeManifest",
    "CreativePolicy",
    "CreativeVariant",
    "DevicePlatform",
    "DeviceType",
    "DeliveryMeasurement",
    "DeliveryMetrics",
    "DeliveryStatus",
    "Disclaimer",
    "DoohMetrics",
    "Error",
    "ErrorCode",
    "EventType",
    "ExtensionObject",
    "FeedFormat",
    "Fonts",
    "Format",
    "FormatCard",
    "FormatCardDetailed",
    "FormatId",
    "FormatIdParameter",
    "Identifier",
    "Input",
    "LandingPageRequirement",
    "Logo",
    "ListCreativesField",
    "MediaBuy",
    "MediaBuyPackage",
    "MediaChannel",
    "Metadata",
    "Offering",
    "OptimizationGoal",
    "OutcomeMeasurement",
    "Overlay",
    "Package",
    "PackageRequest",
    "PackageUpdate",
    "Parameters",
    "Performance",
    "PerformanceFeedback",
    "Placement",
    "Preview",
    "PreviewRender",
    "Pricing",
    "Product",
    "ProductCard",
    "Proposal",
    "ProductCardDetailed",
    "ProductCatalog",
    "ProductFilters",
    "PromotedOfferings",
    "PromotedOfferingsAssetRequirements",
    "PromotedOfferingsRequirement",
    "PromotedProducts",
    "Property",
    "QuartileData",
    "Request",
    "Response",
    "Results",
    "Signal",
    "SignalFilters",
    "SignalPricing",
    "SignalPricingOption",
    "CpmSignalPricingOption",
    "FlatFeeSignalPricingOption",
    "PercentOfMediaSignalPricingOption",
    "Snapshot",
    "SnapshotUnavailableReason",
    "Tags",
    "TargetingOverlay",
    "Transform",
    "VenueBreakdownItem",
    # Pricing types
    "CpaPricingOption",
    "CpcPricingOption",
    "CpcvPricingOption",
    "CpmPricingOption",
    "CppPricingOption",
    "CpvPricingOption",
    "FlatRatePricingOption",
    "PriceGuidance",
    "TimeBasedPricingOption",
    "TimeUnit",
    "VcpmPricingOption",
    # Status enums & simple types
    "SignalCatalogType",
    "Country",
    "CreativeStatus",
    "DaastVersion",
    "DeliverTo",
    "DeliveryType",
    "SortDirection",
    "FeedbackSource",
    "FieldModel",
    "FrequencyCap",
    "FrequencyCapScope",
    "GeoCountry",
    "GeoMetro",
    "GeoPostalArea",
    "GeoRegion",
    "MarkdownFlavor",
    "MeasurementPeriod",
    "MediaBuyStatus",
    "HttpMethod",
    "MetricType",
    "JavascriptModuleType",
    "NotificationType",
    "PreviewOutputFormat",
    "Pacing",
    "Pagination",
    "PaginationRequest",
    "PricingModel",
    "PrimaryCountry",
    "PropertyIdentifierTypes",
    "PropertyType",
    "PublisherDomain",
    "PublisherIdentifierTypes",
    "WebhookResponseType",
    "Responsive",
    "Sort",
    "Status",
    "TaskType",
    "DaastTrackingEvent",
    "VastTrackingEvent",
    "DimensionUnit",
    "UpdateFrequency",
    "UrlAssetType",
    "ValidationMode",
    "VastVersion",
    "ViewThreshold",
    "WcagLevel",
    # Configuration & infrastructure types
    "Authentication",
    "AuthorizedAgents",
    "AvailableMetric",
    "PushNotificationConfig",
    "ReportingCapabilities",
    "ReportingFrequency",
    "ReportingPeriod",
    "ReportingWebhook",
    "AuthenticationScheme",
    "Security",
    # Assets
    "AudioAsset",
    "CssAsset",
    "HtmlAsset",
    "ImageAsset",
    "JavascriptAsset",
    "TextAsset",
    "UrlAsset",
    "VideoAsset",
    "WebhookAsset",
    # Core types
    "AgentConfig",
    "Member",
    "Protocol",
    "ResolvedBrand",
    "ResolvedProperty",
    "TaskResult",
    "WebhookMetadata",
    # Webhook types
    "McpWebhookPayload",
    # Semantic aliases for discriminated unions
    "ActivateSignalErrorResponse",
    "ActivateSignalSuccessResponse",
    "AgentDeployment",
    "AgentDestination",
    "AuthorizedAgent",
    "AuthorizedAgentsByInlineProperties",
    "AuthorizedAgentsByPropertyId",
    "AuthorizedAgentsByPropertyTag",
    "AuthorizedAgentsByPublisherProperties",
    "AuthorizedAgentsBySignalId",
    "AuthorizedAgentsBySignalTag",
    "BothPreviewRender",
    "BuildCreativeErrorResponse",
    "BuildCreativeSuccessResponse",
    "CalibrateContentErrorResponse",
    "CalibrateContentSuccessResponse",
    "CreateContentStandardsErrorResponse",
    "CreateContentStandardsSuccessResponse",
    "CreateMediaBuyErrorResponse",
    "CreateMediaBuySuccessResponse",
    "Deployment",
    "Destination",
    "GetContentStandardsErrorResponse",
    "GetContentStandardsSuccessResponse",
    "GetCreativeDeliveryByBuyerRefRequest",
    "GetCreativeDeliveryByCreativeRequest",
    "GetCreativeDeliveryByMediaBuyRequest",
    "GetMediaBuyArtifactsErrorResponse",
    "GetMediaBuyArtifactsSuccessResponse",
    "GetProductsBriefRequest",
    "GetProductsRefineRequest",
    "GetProductsWholesaleRequest",
    "GetSignalsDiscoveryRequest",
    "GetSignalsLookupRequest",
    "HtmlPreviewRender",
    "InlineDaastAsset",
    "InlineVastAsset",
    "ListContentStandardsErrorResponse",
    "ListContentStandardsSuccessResponse",
    "LogEventErrorResponse",
    "LogEventSuccessResponse",
    "MediaSubAsset",
    "PlatformDeployment",
    "PlatformDestination",
    "PreviewCreativeBatchRequest",
    "PreviewCreativeBatchResponse",
    "PreviewCreativeSingleRequest",
    "PreviewCreativeSingleResponse",
    "PreviewCreativeVariantRequest",
    "PreviewCreativeVariantResponse",
    "PricingOption",
    "PropertyId",
    "PropertyTag",
    "ProvidePerformanceFeedbackByBuyerRefRequest",
    "ProvidePerformanceFeedbackByMediaBuyRequest",
    "ProvidePerformanceFeedbackErrorResponse",
    "ProvidePerformanceFeedbackSuccessResponse",
    "PublisherProperties",
    "PublisherPropertiesAll",
    "PublisherPropertiesById",
    "PublisherPropertiesByTag",
    "SegmentIdActivationKey",
    "KeyValueActivationKey",
    "SiSendTextMessageRequest",
    "SiSendActionResponseRequest",
    "SyncAccountsErrorResponse",
    "SyncAccountsSuccessResponse",
    "SyncCatalogResult",
    "SyncCatalogsErrorResponse",
    "SyncCatalogsSuccessResponse",
    "SyncCreativesErrorResponse",
    "SyncCreativesSuccessResponse",
    "SyncCreativeResult",
    "SyncEventSourcesErrorResponse",
    "SyncEventSourcesSuccessResponse",
    "TextSubAsset",
    "UpdateContentStandardsErrorResponse",
    "UpdateContentStandardsSuccessResponse",
    "UpdateMediaBuyErrorResponse",
    "UpdateMediaBuyPackagesRequest",
    "UpdateMediaBuyPropertiesRequest",
    "UpdateMediaBuySuccessResponse",
    "UrlDaastAsset",
    "UrlPreviewRender",
    "UrlVastAsset",
    "ValidateContentDeliveryErrorResponse",
    "ValidateContentDeliverySuccessResponse",
    # Account financials responses
    "GetAccountFinancialsErrorResponse",
    "GetAccountFinancialsSuccessResponse",
    # Audiences responses
    "MediaBuyDeliveryStatus",
    "SyncAudiencesErrorResponse",
    "SyncAudiencesSuccessResponse",
    # Creative features responses
    "GetCreativeFeaturesErrorResponse",
    "GetCreativeFeaturesSuccessResponse",
    # Internal/special exports
    "GeneratedTaskStatus",
    # Backward compatibility aliases (deprecated)
    "Action",
    "Capability",
    "CoBranding",
    "FormatType",
    "LandingPage",
    "Measurement",
    "Method",
    "ModuleType",
    "TrackingEvent",
    "Unit",
    "OutputFormat",
    "ResponseType",
    "Scheme",
    "SignalType",
    "UrlType",
    "AvailableReportingFrequency",
    # Backward compat: pricing type renames (auction/fixed merged into single types)
    "CpmAuctionPricingOption",
    "CpmFixedRatePricingOption",
    "VcpmAuctionPricingOption",
    "VcpmFixedRatePricingOption",
    # Backward compat: activation key schema change
    "PropertyIdActivationKey",
    "PropertyTagActivationKey",
    # Backward compat: preview alias renames
    "PreviewCreativeFormatRequest",
    "PreviewCreativeManifestRequest",
    "PreviewCreativeStaticResponse",
    "PreviewCreativeInteractiveResponse",
    # Backward compat: types removed from upstream schemas
    "ListAuthorizedPropertiesRequest",
    "ListAuthorizedPropertiesResponse",
    "PackageStatus",
    # Submodules for advanced use:
    "generated",
    "aliases",
]



# generated by datamodel-codegen:
#   filename:  enums/digital_source_type.json
#   timestamp: 2026-02-28T17:39:50+00:00

from __future__ import annotations

from enum import Enum


class DigitalSourceType(Enum):
    digital_capture = 'digital_capture'
    digital_creation = 'digital_creation'
    trained_algorithmic_media = 'trained_algorithmic_media'
    composite_with_trained_algorithmic_media = 'composite_with_trained_algorithmic_media'
    algorithmic_media = 'algorithmic_media'
    composite_capture = 'composite_capture'
    composite_synthetic = 'composite_synthetic'
    human_edits = 'human_edits'
    data_driven_media = 'data_driven_media'



# generated by datamodel-codegen:
#   filename:  content_standards/artifact.json
#   timestamp: 2026-02-28T17:39:50+00:00

from __future__ import annotations

from enum import Enum
from typing import Annotated, Any, Literal

from adcp.types.base import AdCPBaseModel
from pydantic import AnyUrl, AwareDatetime, ConfigDict, Field, RootModel

from ..core import format_id as format_id_1
from ..core import identifier
from ..core import provenance as provenance_1


class Role(Enum):
    title = 'title'
    paragraph = 'paragraph'
    heading = 'heading'
    caption = 'caption'
    quote = 'quote'
    list_item = 'list_item'
    description = 'description'


class TranscriptSource(Enum):
    original_script = 'original_script'
    subtitles = 'subtitles'
    closed_captions = 'closed_captions'
    dub = 'dub'
    generated = 'generated'


class TranscriptSource1(Enum):
    original_script = 'original_script'
    closed_captions = 'closed_captions'
    generated = 'generated'


class Identifiers(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    apple_podcast_id: Annotated[str | None, Field(description='Apple Podcasts ID')] = None
    podcast_guid: Annotated[str | None, Field(description='Podcast GUID (from RSS feed)')] = None
    rss_url: Annotated[AnyUrl | None, Field(description='RSS feed URL')] = None
    spotify_show_id: Annotated[str | None, Field(description='Spotify show ID')] = None
    youtube_video_id: Annotated[str | None, Field(description='YouTube video ID')] = None


class Metadata(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    author: Annotated[str | None, Field(description='Artifact author name')] = None
    canonical: Annotated[AnyUrl | None, Field(description='Canonical URL')] = None
    json_ld: Annotated[
        list[dict[str, Any]] | None, Field(description='JSON-LD structured data (schema.org)')
    ] = None
    keywords: Annotated[str | None, Field(description='Artifact keywords')] = None
    open_graph: Annotated[
        dict[str, Any] | None, Field(description='Open Graph protocol metadata')
    ] = None
    twitter_card: Annotated[dict[str, Any] | None, Field(description='Twitter Card metadata')] = (
        None
    )


class AssetAccess1(AdCPBaseModel):
    method: Literal['bearer_token']
    token: Annotated[str, Field(description='OAuth2 bearer token for Authorization header')]


class Provider(Enum):
    gcp = 'gcp'
    aws = 'aws'


class AssetAccess2(AdCPBaseModel):
    credentials: Annotated[
        dict[str, Any] | None, Field(description='Service account credentials')
    ] = None
    method: Literal['service_account']
    provider: Annotated[Provider, Field(description='Cloud provider')]


class AssetAccess3(AdCPBaseModel):
    method: Literal['signed_url']


class AssetAccess(RootModel[AssetAccess1 | AssetAccess2 | AssetAccess3]):
    root: Annotated[
        AssetAccess1 | AssetAccess2 | AssetAccess3,
        Field(description='Authentication for accessing secured asset URLs'),
    ]


class Assets(AdCPBaseModel):
    content: Annotated[str, Field(description='Text content')]
    heading_level: Annotated[
        int | None, Field(description='Heading level (1-6), only for role=heading', ge=1, le=6)
    ] = None
    language: Annotated[
        str | None,
        Field(
            description="BCP 47 language tag for this text (e.g., 'en', 'es-MX'). Useful when artifact contains mixed-language content."
        ),
    ] = None
    provenance: Annotated[
        provenance_1.Provenance | None,
        Field(description='Provenance for this text block, overrides artifact-level provenance'),
    ] = None
    role: Annotated[
        Role | None,
        Field(
            description="Role of this text in the document. Use 'title' for the main artifact title, 'description' for summaries."
        ),
    ] = None
    type: Literal['text']


class Assets1(AdCPBaseModel):
    access: Annotated[AssetAccess | None, Field(description='Authentication for secured URLs')] = (
        None
    )
    alt_text: Annotated[str | None, Field(description='Alt text or image description')] = None
    caption: Annotated[str | None, Field(description='Image caption')] = None
    height: Annotated[int | None, Field(description='Image height in pixels')] = None
    provenance: Annotated[
        provenance_1.Provenance | None,
        Field(description='Provenance for this image, overrides artifact-level provenance'),
    ] = None
    type: Literal['image']
    url: Annotated[AnyUrl, Field(description='Image URL')]
    width: Annotated[int | None, Field(description='Image width in pixels')] = None


class Assets2(AdCPBaseModel):
    access: Annotated[AssetAccess | None, Field(description='Authentication for secured URLs')] = (
        None
    )
    duration_ms: Annotated[int | None, Field(description='Video duration in milliseconds')] = None
    provenance: Annotated[
        provenance_1.Provenance | None,
        Field(description='Provenance for this video, overrides artifact-level provenance'),
    ] = None
    thumbnail_url: Annotated[AnyUrl | None, Field(description='Video thumbnail URL')] = None
    transcript: Annotated[str | None, Field(description='Video transcript')] = None
    transcript_source: Annotated[
        TranscriptSource | None, Field(description='How the transcript was generated')
    ] = None
    type: Literal['video']
    url: Annotated[AnyUrl, Field(description='Video URL')]


class Assets3(AdCPBaseModel):
    access: Annotated[AssetAccess | None, Field(description='Authentication for secured URLs')] = (
        None
    )
    duration_ms: Annotated[int | None, Field(description='Audio duration in milliseconds')] = None
    provenance: Annotated[
        provenance_1.Provenance | None,
        Field(description='Provenance for this audio, overrides artifact-level provenance'),
    ] = None
    transcript: Annotated[str | None, Field(description='Audio transcript')] = None
    transcript_source: Annotated[
        TranscriptSource1 | None, Field(description='How the transcript was generated')
    ] = None
    type: Literal['audio']
    url: Annotated[AnyUrl, Field(description='Audio URL')]


class Artifact(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    artifact_id: Annotated[
        str,
        Field(
            description="Identifier for this artifact within the property. The property owner defines the scheme (e.g., 'article_12345', 'episode_42_segment_3', 'post_abc123')."
        ),
    ]
    assets: Annotated[
        list[Assets | Assets1 | Assets2 | Assets3],
        Field(
            description='Artifact assets in document flow order - text blocks, images, video, audio'
        ),
    ]
    format_id: Annotated[
        format_id_1.FormatId | None,
        Field(
            description='Optional reference to a format definition. Uses the same format registry as creative formats.'
        ),
    ] = None
    identifiers: Annotated[
        Identifiers | None, Field(description='Platform-specific identifiers for this artifact')
    ] = None
    last_update_time: Annotated[
        AwareDatetime | None,
        Field(description='When the artifact was last modified (ISO 8601 format)'),
    ] = None
    metadata: Annotated[
        Metadata | None, Field(description='Rich metadata extracted from the artifact')
    ] = None
    property_id: Annotated[
        identifier.Identifier,
        Field(description='Identifier for the property where this artifact appears'),
    ]
    provenance: Annotated[
        provenance_1.Provenance | None,
        Field(
            description='Provenance metadata for this artifact. Serves as the default provenance for all assets within this artifact — individual assets can override with their own provenance.'
        ),
    ] = None
    published_time: Annotated[
        AwareDatetime | None, Field(description='When the artifact was published (ISO 8601 format)')
    ] = None
    url: Annotated[
        AnyUrl | None,
        Field(
            description='Optional URL for this artifact (web page, podcast feed, video page). Not all artifacts have URLs (e.g., Instagram content, podcast segments, TV scenes).'
        ),
    ] = None
    variant_id: Annotated[
        str | None,
        Field(
            description="Identifies a specific variant of this artifact. Use for A/B tests, translations, or temporal versions. Examples: 'en', 'es-MX', 'v2', 'headline_test_b'. The combination of artifact_id + variant_id must be unique."
        ),
    ] = None



# generated by datamodel-codegen:
#   filename:  core/signal_pricing_option.json
#   timestamp: 2026-02-28T17:39:50+00:00

from __future__ import annotations

from enum import Enum
from typing import Annotated, Literal

from adcp.types.base import AdCPBaseModel
from pydantic import ConfigDict, Field, RootModel

from . import ext as ext_1


class SignalPricingOption1(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    cpm: Annotated[float, Field(description='Cost per thousand impressions', ge=0.0)]
    currency: Annotated[str, Field(description='ISO 4217 currency code', pattern='^[A-Z]{3}$')]
    ext: ext_1.ExtensionObject | None = None
    model: Literal['cpm']


class SignalPricingOption2(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    currency: Annotated[
        str,
        Field(description='ISO 4217 currency code for the resulting charge', pattern='^[A-Z]{3}$'),
    ]
    ext: ext_1.ExtensionObject | None = None
    max_cpm: Annotated[
        float | None,
        Field(
            description='Optional CPM cap. When set, the effective charge is min(percent × media_spend_per_mille, max_cpm).',
            ge=0.0,
        ),
    ] = None
    model: Literal['percent_of_media']
    percent: Annotated[
        float, Field(description='Percentage of media spend, e.g. 15 = 15%', ge=0.0, le=100.0)
    ]


class Period(Enum):
    monthly = 'monthly'
    quarterly = 'quarterly'
    annual = 'annual'
    campaign = 'campaign'


class SignalPricingOption3(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    amount: Annotated[float, Field(description='Fixed charge for the billing period', ge=0.0)]
    currency: Annotated[str, Field(description='ISO 4217 currency code', pattern='^[A-Z]{3}$')]
    ext: ext_1.ExtensionObject | None = None
    model: Literal['flat_fee']
    period: Annotated[Period, Field(description='Billing period for the flat fee.')]


class SignalPricingOption4(AdCPBaseModel):
    pricing_option_id: Annotated[
        str,
        Field(
            description='Opaque identifier for this pricing option, unique within the signals agent. Pass this in report_usage to identify which pricing option was applied.'
        ),
    ]


class SignalPricingOption5(SignalPricingOption1, SignalPricingOption4):
    pass


class SignalPricingOption6(SignalPricingOption2, SignalPricingOption4):
    pass


class SignalPricingOption7(SignalPricingOption3, SignalPricingOption4):
    pass


class SignalPricingOption(
    RootModel[SignalPricingOption5 | SignalPricingOption6 | SignalPricingOption7]
):
    root: Annotated[
        SignalPricingOption5 | SignalPricingOption6 | SignalPricingOption7,
        Field(
            description='A pricing option offered by a signals agent. Combines pricing_option_id with the signal pricing model fields at the same level — pass pricing_option_id in report_usage for billing verification.',
            title='Signal Pricing Option',
        ),
    ]



from __future__ import annotations

"""Tests for adagents.json validation functionality."""

from unittest.mock import AsyncMock, MagicMock

import pytest

from adcp.adagents import (
    AuthorizationContext,
    _normalize_domain,
    _validate_publisher_domain,
    domain_matches,
    fetch_agent_authorizations,
    get_all_properties,
    get_all_tags,
    get_properties_by_agent,
    identifiers_match,
    verify_agent_authorization,
)
from adcp.exceptions import (
    AdagentsValidationError,
)


def create_mock_httpx_client(mock_response):
    """Helper to create a properly mocked httpx.AsyncClient."""
    mock_get = AsyncMock(return_value=mock_response)
    mock_client_instance = MagicMock()
    mock_client_instance.get = mock_get
    mock_client_instance.__aenter__.return_value = mock_client_instance
    mock_client_instance.__aexit__.return_value = AsyncMock()
    return mock_client_instance


class TestDomainNormalization:
    """Test domain normalization function."""

    def test_normalize_basic(self):
        """Basic normalization should work."""
        assert _normalize_domain("Example.COM") == "example.com"
        assert _normalize_domain("  example.com  ") == "example.com"

    def test_normalize_trailing_slash(self):
        """Should remove trailing slashes."""
        assert _normalize_domain("example.com/") == "example.com"
        assert _normalize_domain("example.com///") == "example.com"

    def test_normalize_trailing_dot(self):
        """Should remove trailing dots."""
        assert _normalize_domain("example.com.") == "example.com"
        assert _normalize_domain("example.com...") == "example.com"

    def test_normalize_both(self):
        """Should remove both trailing slashes and dots."""
        assert _normalize_domain("example.com/.") == "example.com"

    def test_normalize_invalid_double_dots(self):
        """Double dots should raise error."""
        with pytest.raises(AdagentsValidationError, match="Invalid domain format"):
            _normalize_domain("example..com")

    def test_normalize_empty(self):
        """Empty string should raise error."""
        with pytest.raises(AdagentsValidationError, match="Invalid domain format"):
            _normalize_domain("")
        with pytest.raises(AdagentsValidationError, match="Invalid domain format"):
            _normalize_domain("   ")


class TestPublisherDomainValidation:
    """Test publisher domain validation for security."""

    def test_validate_basic(self):
        """Basic valid domains should pass."""
        assert _validate_publisher_domain("example.com") == "example.com"
        assert _validate_publisher_domain("sub.example.com") == "sub.example.com"

    def test_validate_removes_protocol(self):
        """Should strip protocol if present."""
        assert _validate_publisher_domain("https://example.com") == "example.com"
        assert _validate_publisher_domain("http://example.com") == "example.com"

    def test_validate_removes_path(self):
        """Should strip path if present."""
        assert _validate_publisher_domain("example.com/path") == "example.com"
        assert _validate_publisher_domain("https://example.com/path") == "example.com"

    def test_validate_case_insensitive(self):
        """Should normalize to lowercase."""
        assert _validate_publisher_domain("EXAMPLE.COM") == "example.com"

    def test_validate_empty(self):
        """Empty domain should raise error."""
        with pytest.raises(AdagentsValidationError, match="cannot be empty"):
            _validate_publisher_domain("")
        with pytest.raises(AdagentsValidationError, match="cannot be empty"):
            _validate_publisher_domain("   ")

    def test_validate_too_long(self):
        """Domain exceeding DNS max length should raise error."""
        long_domain = "a" * 254
        with pytest.raises(AdagentsValidationError, match="too long"):
            _validate_publisher_domain(long_domain)

    def test_validate_suspicious_chars(self):
        """Suspicious characters should raise error."""
        with pytest.raises(AdagentsValidationError, match="Invalid character"):
            _validate_publisher_domain("example.com\\malicious")
        with pytest.raises(AdagentsValidationError, match="Invalid character"):
            _validate_publisher_domain("user@example.com")
        with pytest.raises(AdagentsValidationError, match="Invalid character"):
            _validate_publisher_domain("example.com with spaces")
        with pytest.raises(AdagentsValidationError, match="Invalid character"):
            _validate_publisher_domain("example.com\n")

    def test_validate_no_dots(self):
        """Domain without dots should raise error."""
        with pytest.raises(AdagentsValidationError, match="must contain at least one dot"):
            _validate_publisher_domain("localhost")


class TestDomainMatching:
    """Test domain matching logic per AdCP spec."""

    def test_exact_match(self):
        """Exact domain match should succeed."""
        assert domain_matches("example.com", "example.com")
        assert domain_matches("sub.example.com", "sub.example.com")

    def test_case_insensitive(self):
        """Domain matching should be case-insensitive."""
        assert domain_matches("Example.com", "example.com")
        assert domain_matches("example.com", "EXAMPLE.COM")

    def test_bare_domain_matches_www(self):
        """Bare domain should match www subdomain."""
        assert domain_matches("www.example.com", "example.com")
        assert domain_matches("m.example.com", "example.com")

    def test_bare_domain_does_not_match_other_subdomains(self):
        """Bare domain should NOT match arbitrary subdomains."""
        assert not domain_matches("api.example.com", "example.com")
        assert not domain_matches("cdn.example.com", "example.com")

    def test_specific_subdomain_does_not_match_others(self):
        """Specific subdomain should only match itself."""
        assert not domain_matches("www.example.com", "api.example.com")
        assert domain_matches("api.example.com", "api.example.com")

    def test_wildcard_matches_all_subdomains(self):
        """Wildcard pattern should match all subdomains."""
        assert domain_matches("www.example.com", "*.example.com")
        assert domain_matches("api.example.com", "*.example.com")
        assert domain_matches("cdn.example.com", "*.example.com")
        assert domain_matches("sub.api.example.com", "*.example.com")

    def test_wildcard_does_not_match_base_domain(self):
        """Wildcard should not match the base domain without subdomain."""
        assert not domain_matches("example.com", "*.example.com")

    def test_no_match_different_domains(self):
        """Different domains should not match."""
        assert not domain_matches("example.com", "other.com")
        assert not domain_matches("www.example.com", "other.com")


class TestIdentifierMatching:
    """Test identifier matching logic."""

    def test_domain_identifier_uses_domain_matching(self):
        """Domain identifiers should use domain matching rules."""
        property_ids = [{"type": "domain", "value": "www.example.com"}]
        agent_ids = [{"type": "domain", "value": "example.com"}]
        assert identifiers_match(property_ids, agent_ids)

    def test_bundle_id_exact_match(self):
        """Bundle IDs require exact match."""
        property_ids = [{"type": "bundle_id", "value": "com.example.app"}]
        agent_ids = [{"type": "bundle_id", "value": "com.example.app"}]
        assert identifiers_match(property_ids, agent_ids)

    def test_bundle_id_no_partial_match(self):
        """Bundle IDs should not partially match."""
        property_ids = [{"type": "bundle_id", "value": "com.example.app"}]
        agent_ids = [{"type": "bundle_id", "value": "com.example"}]
        assert not identifiers_match(property_ids, agent_ids)

    def test_type_mismatch(self):
        """Different identifier types should not match."""
        property_ids = [{"type": "domain", "value": "example.com"}]
        agent_ids = [{"type": "bundle_id", "value": "example.com"}]
        assert not identifiers_match(property_ids, agent_ids)

    def test_multiple_identifiers_any_match(self):
        """Should match if ANY identifier matches."""
        property_ids = [
            {"type": "domain", "value": "example.com"},
            {"type": "bundle_id", "value": "com.example.app"},
        ]
        agent_ids = [{"type": "bundle_id", "value": "com.example.app"}]
        assert identifiers_match(property_ids, agent_ids)

    def test_no_match_empty_lists(self):
        """Empty lists should not match."""
        assert not identifiers_match([], [])
        assert not identifiers_match([{"type": "domain", "value": "example.com"}], [])


class TestVerifyAgentAuthorization:
    """Test agent authorization verification."""

    def test_agent_authorized_no_properties_restriction(self):
        """Agent with empty properties array is authorized for all properties."""
        adagents_data = {
            "authorized_agents": [{"url": "https://sales-agent.example.com", "properties": []}]
        }
        assert verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", None, None
        )

    def test_agent_authorized_no_properties_field(self):
        """Agent without properties field is authorized for all properties."""
        adagents_data = {"authorized_agents": [{"url": "https://sales-agent.example.com"}]}
        assert verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", None, None
        )

    def test_agent_url_protocol_agnostic(self):
        """Agent URL matching should ignore protocol."""
        adagents_data = {"authorized_agents": [{"url": "https://sales-agent.example.com"}]}
        assert verify_agent_authorization(
            adagents_data, "http://sales-agent.example.com", None, None
        )

    def test_agent_url_trailing_slash_ignored(self):
        """Agent URL matching should ignore trailing slash."""
        adagents_data = {"authorized_agents": [{"url": "https://sales-agent.example.com/"}]}
        assert verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", None, None
        )

    def test_agent_authorized_specific_property(self):
        """Agent authorized for specific property type and identifiers."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://sales-agent.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Example Site",
                            "identifiers": [{"type": "domain", "value": "example.com"}],
                        }
                    ],
                }
            ]
        }
        assert verify_agent_authorization(
            adagents_data,
            "https://sales-agent.example.com",
            "website",
            [{"type": "domain", "value": "www.example.com"}],
        )

    def test_agent_not_authorized_wrong_property_type(self):
        """Agent should not be authorized for wrong property type."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://sales-agent.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "identifiers": [{"type": "domain", "value": "example.com"}],
                        }
                    ],
                }
            ]
        }
        assert not verify_agent_authorization(
            adagents_data,
            "https://sales-agent.example.com",
            "mobile_app",
            [{"type": "domain", "value": "example.com"}],
        )

    def test_agent_not_authorized_wrong_identifier(self):
        """Agent should not be authorized for wrong identifier."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://sales-agent.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "identifiers": [{"type": "domain", "value": "example.com"}],
                        }
                    ],
                }
            ]
        }
        assert not verify_agent_authorization(
            adagents_data,
            "https://sales-agent.example.com",
            "website",
            [{"type": "domain", "value": "other.com"}],
        )

    def test_agent_not_in_list(self):
        """Agent not in authorized_agents list should not be authorized."""
        adagents_data = {
            "authorized_agents": [{"url": "https://other-agent.example.com", "properties": []}]
        }
        assert not verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", None, None
        )

    def test_multiple_agents(self):
        """Should find correct agent in list."""
        adagents_data = {
            "authorized_agents": [
                {"url": "https://agent1.example.com", "properties": []},
                {"url": "https://agent2.example.com", "properties": []},
                {"url": "https://sales-agent.example.com", "properties": []},
            ]
        }
        assert verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", None, None
        )

    def test_invalid_adagents_data_not_dict(self):
        """Should raise error if adagents_data is not a dict."""
        with pytest.raises(AdagentsValidationError, match="must be a dictionary"):
            verify_agent_authorization([], "https://agent.example.com", None, None)

    def test_invalid_adagents_data_no_authorized_agents(self):
        """Should raise error if authorized_agents field is missing."""
        with pytest.raises(AdagentsValidationError, match="authorized_agents"):
            verify_agent_authorization({}, "https://agent.example.com", None, None)

    def test_invalid_authorized_agents_not_list(self):
        """Should raise error if authorized_agents is not a list."""
        with pytest.raises(AdagentsValidationError, match="authorized_agents"):
            verify_agent_authorization(
                {"authorized_agents": "not a list"}, "https://agent.example.com", None, None
            )

    def test_property_type_match_without_identifiers(self):
        """Should match property type even without identifier check."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://sales-agent.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "identifiers": [{"type": "domain", "value": "example.com"}],
                        }
                    ],
                }
            ]
        }
        # When property_identifiers is None, just check property_type
        assert verify_agent_authorization(
            adagents_data, "https://sales-agent.example.com", "website", None
        )


class TestFetchAdagents:
    """Test fetching adagents.json from publisher domains."""

    @pytest.mark.asyncio
    async def test_fetch_success(self):
        """Should successfully fetch and parse adagents.json."""
        from adcp.adagents import fetch_adagents

        mock_adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent.example.com",
                    "authorized_for": "All properties",
                    "authorization_type": "property_ids",
                    "property_ids": ["site1", "site2"],
                }
            ]
        }

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = mock_adagents_data
        mock_response.raise_for_status = MagicMock()

        mock_client = create_mock_httpx_client(mock_response)

        result = await fetch_adagents("example.com", client=mock_client)

        assert result == mock_adagents_data
        mock_client.get.assert_called_once()
        call_args = mock_client.get.call_args
        assert "https://example.com/.well-known/adagents.json" in str(call_args)

    @pytest.mark.asyncio
    async def test_fetch_follows_authoritative_location(self):
        """Should follow authoritative_location redirect and return resolved data."""
        from adcp.adagents import fetch_adagents

        # Initial response has authoritative_location redirect
        redirect_response_data = {
            "$schema": "/schemas/2.6.0/adagents.json",
            "authoritative_location": "https://cdn.example.com/adagents/v2/adagents.json",
            "last_updated": "2025-01-15T10:00:00Z",
        }

        # Final resolved data at the authoritative location
        resolved_data = {
            "$schema": "/schemas/2.6.0/adagents.json",
            "authorized_agents": [
                {
                    "url": "https://agent.example.com",
                    "authorized_for": "All properties",
                    "authorization_type": "property_tags",
                    "property_tags": ["all"],
                }
            ],
            "last_updated": "2025-01-15T10:00:00Z",
        }

        # Mock client that returns different responses based on URL
        called_urls: list[str] = []
        responses = [redirect_response_data, resolved_data]

        async def mock_get(url, **kwargs):
            called_urls.append(url)
            mock_response = MagicMock()
            mock_response.status_code = 200
            mock_response.json.return_value = responses[len(called_urls) - 1]
            return mock_response

        mock_client = MagicMock()
        mock_client.get = mock_get

        result = await fetch_adagents("example.com", client=mock_client)

        assert result == resolved_data
        assert called_urls == [
            "https://example.com/.well-known/adagents.json",
            "https://cdn.example.com/adagents/v2/adagents.json",
        ]

    @pytest.mark.asyncio
    async def test_fetch_rejects_non_https_authoritative_location(self):
        """Should reject authoritative_location that uses HTTP instead of HTTPS."""
        from adcp.adagents import fetch_adagents

        redirect_response_data = {
            "$schema": "/schemas/2.6.0/adagents.json",
            "authoritative_location": "http://cdn.example.com/adagents.json",  # HTTP not HTTPS
            "last_updated": "2025-01-15T10:00:00Z",
        }

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = redirect_response_data

        mock_client = create_mock_httpx_client(mock_response)

        with pytest.raises(AdagentsValidationError, match="HTTPS"):
            await fetch_adagents("example.com", client=mock_client)

    @pytest.mark.asyncio
    async def test_fetch_prevents_redirect_loop(self):
        """Should detect and prevent circular redirect loops."""
        from adcp.adagents import fetch_adagents

        # Circular redirect: A -> B -> A
        redirect_data = {
            "$schema": "/schemas/2.6.0/adagents.json",
            "authoritative_location": "https://example.com/.well-known/adagents.json",
            "last_updated": "2025-01-15T10:00:00Z",
        }

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = redirect_data

        mock_client = create_mock_httpx_client(mock_response)

        with pytest.raises(AdagentsValidationError, match="redirect loop|already visited"):
            await fetch_adagents("example.com", client=mock_client)

    @pytest.mark.asyncio
    async def test_fetch_enforces_max_redirect_depth(self):
        """Should enforce maximum redirect depth to prevent abuse."""
        from adcp.adagents import fetch_adagents

        # Create a long chain of redirects
        call_count = [0]

        async def mock_get(url, **kwargs):
            call_count[0] += 1
            mock_response = MagicMock()
            mock_response.status_code = 200
            # Always return a redirect to a new URL
            mock_response.json.return_value = {
                "$schema": "/schemas/2.6.0/adagents.json",
                "authoritative_location": f"https://cdn{call_count[0]}.example.com/adagents.json",
                "last_updated": "2025-01-15T10:00:00Z",
            }
            return mock_response

        mock_client = MagicMock()
        mock_client.get = mock_get

        with pytest.raises(AdagentsValidationError, match="redirect|depth"):
            await fetch_adagents("example.com", client=mock_client)

        # Should stop after reasonable number of redirects (not go forever)
        assert call_count[0] <= 10


class TestVerifyAgentForProperty:
    """Test convenience wrapper for fetching and verifying in one call."""

    @pytest.mark.asyncio
    async def test_verify_success(self):
        """Should fetch and verify authorization successfully."""
        from adcp.adagents import verify_agent_for_property

        mock_adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent.example.com",
                    "authorized_for": "All properties",
                    "authorization_type": "property_ids",
                    "property_ids": ["site1", "site2"],
                }
            ]
        }

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = mock_adagents_data
        mock_response.raise_for_status = MagicMock()

        mock_client = create_mock_httpx_client(mock_response)

        # Verify authorized agent
        result = await verify_agent_for_property(
            publisher_domain="example.com",
            agent_url="https://agent.example.com",
            property_identifiers=[{"type": "property_id", "value": "site1"}],
            client=mock_client,
        )

        assert result is True
        mock_client.get.assert_called_once()


class TestGetAllProperties:
    """Test extracting all properties from adagents.json data."""

    def test_get_all_properties(self):
        """Should extract all properties from all agents."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        },
                        {
                            "property_type": "mobile_app",
                            "name": "App 1",
                            "identifiers": [{"type": "bundle_id", "value": "com.site1.app"}],
                        },
                    ],
                },
                {
                    "url": "https://agent2.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 2",
                            "identifiers": [{"type": "domain", "value": "site2.com"}],
                        }
                    ],
                },
            ]
        }

        properties = get_all_properties(adagents_data)
        assert len(properties) == 3
        assert properties[0]["name"] == "Site 1"
        assert properties[0]["agent_url"] == "https://agent1.example.com"
        assert properties[1]["name"] == "App 1"
        assert properties[1]["agent_url"] == "https://agent1.example.com"
        assert properties[2]["name"] == "Site 2"
        assert properties[2]["agent_url"] == "https://agent2.example.com"

    def test_get_all_properties_with_empty_properties(self):
        """Should handle agents with empty properties array."""
        adagents_data = {
            "authorized_agents": [
                {"url": "https://agent1.example.com", "properties": []},
                {
                    "url": "https://agent2.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site",
                            "identifiers": [{"type": "domain", "value": "site.com"}],
                        }
                    ],
                },
            ]
        }

        properties = get_all_properties(adagents_data)
        assert len(properties) == 1
        assert properties[0]["name"] == "Site"

    def test_get_all_properties_invalid_data(self):
        """Should raise error for invalid data."""
        with pytest.raises(AdagentsValidationError):
            get_all_properties([])


class TestGetAllTags:
    """Test extracting all unique tags from adagents.json data."""

    def test_get_all_tags(self):
        """Should extract all unique tags from properties."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                            "tags": ["premium", "news"],
                        },
                        {
                            "property_type": "mobile_app",
                            "name": "App 1",
                            "identifiers": [{"type": "bundle_id", "value": "com.site1.app"}],
                            "tags": ["mobile", "premium"],
                        },
                    ],
                },
                {
                    "url": "https://agent2.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 2",
                            "identifiers": [{"type": "domain", "value": "site2.com"}],
                            "tags": ["sports"],
                        }
                    ],
                },
            ]
        }

        tags = get_all_tags(adagents_data)
        assert tags == {"premium", "news", "mobile", "sports"}

    def test_get_all_tags_no_tags(self):
        """Should return empty set when no tags present."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        }
                    ],
                }
            ]
        }

        tags = get_all_tags(adagents_data)
        assert tags == set()


class TestGetPropertiesByAgent:
    """Test getting properties for a specific agent."""

    def test_get_properties_by_agent_inline_properties(self):
        """Should return inline properties for agent with authorization_type=inline_properties."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test properties",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        },
                        {
                            "property_type": "mobile_app",
                            "name": "App 1",
                            "identifiers": [{"type": "bundle_id", "value": "com.site1.app"}],
                        },
                    ],
                },
            ]
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 2
        assert properties[0]["name"] == "Site 1"
        assert properties[1]["name"] == "App 1"

    def test_get_properties_by_agent_legacy_properties(self):
        """Should return properties for agent without explicit authorization_type."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        },
                    ],
                },
            ]
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 1
        assert properties[0]["name"] == "Site 1"

    def test_get_properties_by_agent_property_ids(self):
        """Should filter top-level properties by property_id for authorization_type=property_ids."""
        adagents_data = {
            "properties": [
                {
                    "property_id": "site1",
                    "property_type": "website",
                    "name": "Site 1",
                    "identifiers": [{"type": "domain", "value": "site1.com"}],
                },
                {
                    "property_id": "site2",
                    "property_type": "website",
                    "name": "Site 2",
                    "identifiers": [{"type": "domain", "value": "site2.com"}],
                },
                {
                    "property_id": "site3",
                    "property_type": "website",
                    "name": "Site 3",
                    "identifiers": [{"type": "domain", "value": "site3.com"}],
                },
            ],
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "property_ids",
                    "authorized_for": "Selected properties",
                    "property_ids": ["site1", "site3"],
                },
            ],
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 2
        assert properties[0]["name"] == "Site 1"
        assert properties[1]["name"] == "Site 3"

    def test_get_properties_by_agent_property_tags(self):
        """Should filter top-level properties by tags for authorization_type=property_tags."""
        adagents_data = {
            "properties": [
                {
                    "property_id": "site1",
                    "property_type": "website",
                    "name": "Site 1",
                    "identifiers": [{"type": "domain", "value": "site1.com"}],
                    "tags": ["premium", "news"],
                },
                {
                    "property_id": "site2",
                    "property_type": "website",
                    "name": "Site 2",
                    "identifiers": [{"type": "domain", "value": "site2.com"}],
                    "tags": ["sports"],
                },
                {
                    "property_id": "site3",
                    "property_type": "website",
                    "name": "Site 3",
                    "identifiers": [{"type": "domain", "value": "site3.com"}],
                    "tags": ["premium", "entertainment"],
                },
            ],
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "property_tags",
                    "authorized_for": "Premium properties",
                    "property_tags": ["premium"],
                },
            ],
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 2
        assert properties[0]["name"] == "Site 1"
        assert properties[1]["name"] == "Site 3"

    def test_get_properties_by_agent_property_tags_multiple(self):
        """Should match properties with any of the authorized tags."""
        adagents_data = {
            "properties": [
                {
                    "property_id": "site1",
                    "property_type": "website",
                    "name": "Site 1",
                    "identifiers": [{"type": "domain", "value": "site1.com"}],
                    "tags": ["news"],
                },
                {
                    "property_id": "site2",
                    "property_type": "website",
                    "name": "Site 2",
                    "identifiers": [{"type": "domain", "value": "site2.com"}],
                    "tags": ["sports"],
                },
                {
                    "property_id": "site3",
                    "property_type": "website",
                    "name": "Site 3",
                    "identifiers": [{"type": "domain", "value": "site3.com"}],
                    "tags": ["entertainment"],
                },
            ],
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "property_tags",
                    "authorized_for": "News and sports",
                    "property_tags": ["news", "sports"],
                },
            ],
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 2
        assert properties[0]["name"] == "Site 1"
        assert properties[1]["name"] == "Site 2"

    def test_get_properties_by_agent_publisher_properties(self):
        """Should return publisher_properties selectors for publisher_properties type."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "publisher_properties",
                    "authorized_for": "Cross-domain properties",
                    "publisher_properties": [
                        {
                            "publisher_domain": "cnn.com",
                            "selection_type": "by_tag",
                            "property_tags": ["ctv"],
                        },
                        {
                            "publisher_domain": "espn.com",
                            "selection_type": "all",
                        },
                    ],
                },
            ],
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 2
        assert properties[0]["publisher_domain"] == "cnn.com"
        assert properties[0]["selection_type"] == "by_tag"
        assert properties[1]["publisher_domain"] == "espn.com"
        assert properties[1]["selection_type"] == "all"

    def test_get_properties_by_agent_protocol_agnostic(self):
        """Should match agent URL regardless of protocol."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        }
                    ],
                }
            ]
        }

        properties = get_properties_by_agent(adagents_data, "http://agent1.example.com")
        assert len(properties) == 1
        assert properties[0]["name"] == "Site 1"

    def test_get_properties_by_agent_not_found(self):
        """Should return empty list for unknown agent."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "site1.com"}],
                        }
                    ],
                }
            ]
        }

        properties = get_properties_by_agent(adagents_data, "https://unknown-agent.com")
        assert len(properties) == 0

    def test_get_properties_by_agent_no_top_level_properties(self):
        """Should return empty list when using property_ids/tags but no top-level props."""
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://agent1.example.com",
                    "authorization_type": "property_ids",
                    "authorized_for": "Test",
                    "property_ids": ["site1"],
                },
            ],
        }

        properties = get_properties_by_agent(adagents_data, "https://agent1.example.com")
        assert len(properties) == 0


class TestAuthorizationContext:
    """Test AuthorizationContext class."""

    def test_extract_property_ids(self):
        """Should extract property IDs from properties using property_id field."""
        properties = [
            {
                "property_id": "prop1",
                "property_type": "website",
                "name": "Site 1",
                "identifiers": [{"type": "domain", "value": "site1.com"}],
            },
            {
                "property_id": "prop2",
                "property_type": "mobile_app",
                "name": "App 1",
                "identifiers": [{"type": "bundle_id", "value": "com.site1.app"}],
            },
        ]

        ctx = AuthorizationContext(properties)
        assert ctx.property_ids == ["prop1", "prop2"]

    def test_extract_property_tags(self):
        """Should extract unique tags from properties."""
        properties = [
            {
                "property_id": "prop1",
                "property_type": "website",
                "name": "Site 1",
                "tags": ["premium", "news"],
            },
            {
                "property_id": "prop2",
                "property_type": "website",
                "name": "Site 2",
                "tags": ["premium", "sports"],
            },
        ]

        ctx = AuthorizationContext(properties)
        assert set(ctx.property_tags) == {"premium", "news", "sports"}

    def test_deduplicate_tags(self):
        """Should deduplicate tags."""
        properties = [
            {
                "property_id": "prop1",
                "tags": ["premium", "news"],
            },
            {
                "property_id": "prop2",
                "tags": ["premium", "sports"],
            },
        ]

        ctx = AuthorizationContext(properties)
        # Each tag should appear only once
        assert ctx.property_tags.count("premium") == 1

    def test_handle_missing_fields(self):
        """Should handle properties without property_id or tags."""
        properties = [
            {
                "property_type": "website",
                "name": "Site 1",
            }
        ]

        ctx = AuthorizationContext(properties)
        assert ctx.property_ids == []
        assert ctx.property_tags == []

    def test_raw_properties_preserved(self):
        """Should preserve raw properties data."""
        properties = [
            {
                "property_id": "prop1",
                "property_type": "website",
                "name": "Site 1",
                "custom_field": "custom_value",
            }
        ]

        ctx = AuthorizationContext(properties)
        assert ctx.raw_properties == properties
        assert ctx.raw_properties[0]["custom_field"] == "custom_value"

    def test_repr(self):
        """Should have useful string representation."""
        properties = [
            {
                "property_id": "prop1",
                "tags": ["premium"],
            }
        ]

        ctx = AuthorizationContext(properties)
        repr_str = repr(ctx)
        assert "AuthorizationContext" in repr_str
        assert "property_ids" in repr_str
        assert "property_tags" in repr_str


@pytest.mark.asyncio
class TestFetchAgentAuthorizations:
    """Test fetch_agent_authorizations function."""

    async def test_single_publisher_authorized(self):
        """Should return authorization context for authorized publisher."""
        from unittest.mock import patch

        # Mock adagents.json data
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                            "tags": ["premium", "news"],
                        }
                    ],
                }
            ]
        }

        # Mock fetch_adagents to return our test data
        with patch("adcp.adagents.fetch_adagents", new_callable=AsyncMock) as mock_fetch:
            mock_fetch.return_value = adagents_data

            contexts = await fetch_agent_authorizations("https://our-agent.com", ["nytimes.com"])

            assert len(contexts) == 1
            assert "nytimes.com" in contexts
            ctx = contexts["nytimes.com"]
            assert ctx.property_ids == ["prop1"]
            assert "premium" in ctx.property_tags
            assert "news" in ctx.property_tags

    async def test_multiple_publishers(self):
        """Should fetch and return contexts for multiple publishers in parallel."""
        from unittest.mock import patch

        # Mock adagents.json data for different publishers
        nytimes_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "nyt_prop1",
                            "property_type": "website",
                            "name": "NYT Site",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                            "tags": ["news"],
                        }
                    ],
                }
            ]
        }

        wsj_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "wsj_prop1",
                            "property_type": "website",
                            "name": "WSJ Site",
                            "identifiers": [{"type": "domain", "value": "wsj.com"}],
                            "tags": ["business"],
                        }
                    ],
                }
            ]
        }

        async def mock_fetch_adagents(domain, **kwargs):
            if domain == "nytimes.com":
                return nytimes_data
            elif domain == "wsj.com":
                return wsj_data
            else:
                raise Exception("Unexpected domain")

        with patch("adcp.adagents.fetch_adagents", side_effect=mock_fetch_adagents):
            contexts = await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com", "wsj.com"]
            )

            assert len(contexts) == 2
            assert "nytimes.com" in contexts
            assert "wsj.com" in contexts
            assert contexts["nytimes.com"].property_ids == ["nyt_prop1"]
            assert contexts["wsj.com"].property_ids == ["wsj_prop1"]

    async def test_skip_unauthorized_publishers(self):
        """Should skip publishers where agent is not authorized."""
        from unittest.mock import patch

        # nytimes authorizes our agent
        nytimes_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                        }
                    ],
                }
            ]
        }

        # wsj does NOT authorize our agent
        wsj_data = {
            "authorized_agents": [
                {
                    "url": "https://different-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop2",
                            "property_type": "website",
                            "name": "Site 2",
                            "identifiers": [{"type": "domain", "value": "wsj.com"}],
                        }
                    ],
                }
            ]
        }

        async def mock_fetch_adagents(domain, **kwargs):
            if domain == "nytimes.com":
                return nytimes_data
            elif domain == "wsj.com":
                return wsj_data
            else:
                raise Exception("Unexpected domain")

        with patch("adcp.adagents.fetch_adagents", side_effect=mock_fetch_adagents):
            contexts = await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com", "wsj.com"]
            )

            # Should only include nytimes
            assert len(contexts) == 1
            assert "nytimes.com" in contexts
            assert "wsj.com" not in contexts

    async def test_skip_missing_adagents_json(self):
        """Should silently skip publishers with missing adagents.json."""
        from unittest.mock import patch

        from adcp.exceptions import AdagentsNotFoundError

        # nytimes has adagents.json
        nytimes_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                        }
                    ],
                }
            ]
        }

        async def mock_fetch_adagents(domain, **kwargs):
            if domain == "nytimes.com":
                return nytimes_data
            elif domain == "wsj.com":
                # wsj doesn't have adagents.json (404)
                raise AdagentsNotFoundError("wsj.com")
            else:
                raise Exception("Unexpected domain")

        with patch("adcp.adagents.fetch_adagents", side_effect=mock_fetch_adagents):
            contexts = await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com", "wsj.com"]
            )

            # Should only include nytimes
            assert len(contexts) == 1
            assert "nytimes.com" in contexts
            assert "wsj.com" not in contexts

    async def test_skip_invalid_adagents_json(self):
        """Should silently skip publishers with invalid adagents.json."""
        from unittest.mock import patch

        from adcp.exceptions import AdagentsValidationError

        nytimes_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                        }
                    ],
                }
            ]
        }

        async def mock_fetch_adagents(domain, **kwargs):
            if domain == "nytimes.com":
                return nytimes_data
            elif domain == "wsj.com":
                # wsj has invalid adagents.json
                raise AdagentsValidationError("Invalid JSON")
            else:
                raise Exception("Unexpected domain")

        with patch("adcp.adagents.fetch_adagents", side_effect=mock_fetch_adagents):
            contexts = await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com", "wsj.com"]
            )

            # Should only include nytimes
            assert len(contexts) == 1
            assert "nytimes.com" in contexts
            assert "wsj.com" not in contexts

    async def test_empty_result_when_no_authorizations(self):
        """Should return empty dict when no publishers authorize the agent."""
        from unittest.mock import patch

        # No publishers authorize our agent
        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://different-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "example.com"}],
                        }
                    ],
                }
            ]
        }

        with patch("adcp.adagents.fetch_adagents", new_callable=AsyncMock) as mock_fetch:
            mock_fetch.return_value = adagents_data

            contexts = await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com", "wsj.com"]
            )

            assert len(contexts) == 0
            assert contexts == {}

    async def test_uses_provided_http_client(self):
        """Should use provided HTTP client for connection pooling."""
        from unittest.mock import MagicMock, patch

        import httpx

        adagents_data = {
            "authorized_agents": [
                {
                    "url": "https://our-agent.com",
                    "authorization_type": "inline_properties",
                    "authorized_for": "Test",
                    "properties": [
                        {
                            "property_id": "prop1",
                            "property_type": "website",
                            "name": "Site 1",
                            "identifiers": [{"type": "domain", "value": "nytimes.com"}],
                        }
                    ],
                }
            ]
        }

        mock_client = MagicMock(spec=httpx.AsyncClient)

        with patch("adcp.adagents.fetch_adagents", new_callable=AsyncMock) as mock_fetch:
            mock_fetch.return_value = adagents_data

            await fetch_agent_authorizations(
                "https://our-agent.com", ["nytimes.com"], client=mock_client
            )

            # Verify fetch_adagents was called with the provided client
            mock_fetch.assert_called_once()
            call_kwargs = mock_fetch.call_args[1]
            assert call_kwargs.get("client") == mock_client



# generated by datamodel-codegen:
#   filename:  core/async_response_data.json
#   timestamp: 2026-02-23T01:56:40+00:00

from __future__ import annotations

from typing import Annotated

from pydantic import Field, RootModel

from ..media_buy import (
    create_media_buy_async_response_input_required,
    create_media_buy_async_response_submitted,
    create_media_buy_async_response_working,
    create_media_buy_response,
    get_products_async_response_input_required,
    get_products_async_response_submitted,
    get_products_async_response_working,
    get_products_response,
    sync_catalogs_async_response_input_required,
    sync_catalogs_async_response_submitted,
    sync_catalogs_async_response_working,
    sync_catalogs_response,
    sync_creatives_async_response_input_required,
    sync_creatives_async_response_submitted,
    sync_creatives_async_response_working,
    sync_creatives_response,
    update_media_buy_async_response_input_required,
    update_media_buy_async_response_submitted,
    update_media_buy_async_response_working,
    update_media_buy_response,
)


class AdcpAsyncResponseData(
    RootModel[
        get_products_response.GetProductsResponse
        | get_products_async_response_working.GetProductsWorking
        | get_products_async_response_input_required.GetProductsInputRequired
        | get_products_async_response_submitted.GetProductsSubmitted
        | create_media_buy_response.CreateMediaBuyResponse
        | create_media_buy_async_response_working.CreateMediaBuyWorking
        | create_media_buy_async_response_input_required.CreateMediaBuyInputRequired
        | create_media_buy_async_response_submitted.CreateMediaBuySubmitted
        | update_media_buy_response.UpdateMediaBuyResponse
        | update_media_buy_async_response_working.UpdateMediaBuyWorking
        | update_media_buy_async_response_input_required.UpdateMediaBuyInputRequired
        | update_media_buy_async_response_submitted.UpdateMediaBuySubmitted
        | sync_creatives_response.SyncCreativesResponse
        | sync_creatives_async_response_working.SyncCreativesWorking
        | sync_creatives_async_response_input_required.SyncCreativesInputRequired
        | sync_creatives_async_response_submitted.SyncCreativesSubmitted
        | sync_catalogs_response.SyncCatalogsResponse
        | sync_catalogs_async_response_working.SyncCatalogsWorking
        | sync_catalogs_async_response_input_required.SyncCatalogsInputRequired
        | sync_catalogs_async_response_submitted.SyncCatalogsSubmitted
    ]
):
    root: Annotated[
        get_products_response.GetProductsResponse
        | get_products_async_response_working.GetProductsWorking
        | get_products_async_response_input_required.GetProductsInputRequired
        | get_products_async_response_submitted.GetProductsSubmitted
        | create_media_buy_response.CreateMediaBuyResponse
        | create_media_buy_async_response_working.CreateMediaBuyWorking
        | create_media_buy_async_response_input_required.CreateMediaBuyInputRequired
        | create_media_buy_async_response_submitted.CreateMediaBuySubmitted
        | update_media_buy_response.UpdateMediaBuyResponse
        | update_media_buy_async_response_working.UpdateMediaBuyWorking
        | update_media_buy_async_response_input_required.UpdateMediaBuyInputRequired
        | update_media_buy_async_response_submitted.UpdateMediaBuySubmitted
        | sync_creatives_response.SyncCreativesResponse
        | sync_creatives_async_response_working.SyncCreativesWorking
        | sync_creatives_async_response_input_required.SyncCreativesInputRequired
        | sync_creatives_async_response_submitted.SyncCreativesSubmitted
        | sync_catalogs_response.SyncCatalogsResponse
        | sync_catalogs_async_response_working.SyncCatalogsWorking
        | sync_catalogs_async_response_input_required.SyncCatalogsInputRequired
        | sync_catalogs_async_response_submitted.SyncCatalogsSubmitted,
        Field(
            description='Union of all possible data payloads for async task webhook responses. For completed/failed statuses, use the main task response schema. For working/input-required/submitted, use the status-specific schemas.',
            title='AdCP Async Response Data',
        ),
    ]



# generated by datamodel-codegen:
#   filename:  enums/validation_mode.json
#   timestamp: 2026-02-23T01:56:40+00:00

from __future__ import annotations

from enum import Enum


class ValidationMode(Enum):
    strict = 'strict'
    lenient = 'lenient'



# generated by datamodel-codegen:
#   filename:  core/product.json
#   timestamp: 2026-02-28T17:39:50+00:00

from __future__ import annotations

from enum import Enum
from typing import Annotated, Any

from adcp.types.base import AdCPBaseModel
from pydantic import AwareDatetime, ConfigDict, Field, RootModel

from ..enums import action_source, catalog_type
from ..enums import channels as channels_1
from ..enums import delivery_type as delivery_type_1
from ..enums import reach_unit
from . import creative_policy as creative_policy_1
from . import data_provider_signal_selector, delivery_forecast
from . import ext as ext_1
from . import format_id as format_id_1
from . import outcome_measurement as outcome_measurement_1
from . import placement, pricing_option, publisher_property_selector
from . import reporting_capabilities as reporting_capabilities_1


class MatchedGtin(RootModel[str]):
    root: Annotated[str, Field(pattern='^[0-9]{8,14}$')]


class CatalogMatch(AdCPBaseModel):
    matched_count: Annotated[
        int | None,
        Field(description="Number of catalog items that matched this product's inventory.", ge=0),
    ] = None
    matched_gtins: Annotated[
        list[MatchedGtin] | None,
        Field(
            description="GTINs from the buyer's catalog that are eligible on this product's inventory. Standard GTIN formats (GTIN-8 through GTIN-14). Only present for product-type catalogs with GTIN matching."
        ),
    ] = None
    matched_ids: Annotated[
        list[str] | None,
        Field(
            description="Item IDs from the buyer's catalog that matched this product's inventory. The ID type depends on the catalog type and content_id_type (e.g., SKUs for product catalogs, job_ids for job catalogs, offering_ids for offering catalogs)."
        ),
    ] = None
    submitted_count: Annotated[
        int, Field(description="Total catalog items evaluated from the buyer's catalog.", ge=0)
    ]


class SupportedTarget(Enum):
    cost_per = 'cost_per'
    per_ad_spend = 'per_ad_spend'
    maximize_value = 'maximize_value'


class ConversionTracking(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    action_sources: Annotated[
        list[action_source.ActionSource] | None,
        Field(
            description="Action sources relevant to this product (e.g. a retail media product might have 'in_store' and 'website', while a display product might only have 'website')",
            min_length=1,
        ),
    ] = None
    platform_managed: Annotated[
        bool | None,
        Field(
            description="Whether the seller provides its own always-on measurement (e.g. Amazon sales attribution for Amazon advertisers). When true, sync_event_sources response will include seller-managed event sources with managed_by='seller'."
        ),
    ] = None
    supported_targets: Annotated[
        list[SupportedTarget] | None,
        Field(
            description='Target kinds available for event goals on this product. Values match target.kind on the optimization goal. cost_per: target cost per conversion event. per_ad_spend: target return on ad spend (requires value_field on event sources). maximize_value: maximize total conversion value without a specific ratio target (requires value_field). Only these target kinds are accepted — goals with unlisted target kinds will be rejected. A goal without a target implicitly maximizes conversion count within budget — no declaration needed for that mode. When omitted, buyers can still set target-less event goals.',
            min_length=1,
        ),
    ] = None


class DeliveryMeasurement(AdCPBaseModel):
    notes: Annotated[
        str | None,
        Field(
            description="Additional details about measurement methodology in plain language (e.g., 'MRC-accredited viewability. 50% in-view for 1s display / 2s video', 'Panel-based demographic measurement updated monthly')"
        ),
    ] = None
    provider: Annotated[
        str,
        Field(
            description="Measurement provider(s) used for this product (e.g., 'Google Ad Manager with IAS viewability', 'Nielsen DAR', 'Geopath for DOOH impressions')"
        ),
    ]


class SupportedMetric(Enum):
    clicks = 'clicks'
    views = 'views'
    completed_views = 'completed_views'
    viewed_seconds = 'viewed_seconds'
    attention_seconds = 'attention_seconds'
    attention_score = 'attention_score'
    engagements = 'engagements'
    follows = 'follows'
    saves = 'saves'
    profile_visits = 'profile_visits'
    reach = 'reach'


class SupportedTarget1(Enum):
    cost_per = 'cost_per'
    threshold_rate = 'threshold_rate'


class SupportedViewDuration(RootModel[float]):
    root: Annotated[float, Field(gt=0.0)]


class ProductCard(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    format_id: Annotated[
        format_id_1.FormatId,
        Field(
            description='Creative format defining the card layout (typically product_card_standard)'
        ),
    ]
    manifest: Annotated[
        dict[str, Any],
        Field(description='Asset manifest for rendering the card, structure defined by the format'),
    ]


class ProductCardDetailed(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    format_id: Annotated[
        format_id_1.FormatId,
        Field(
            description='Creative format defining the detailed card layout (typically product_card_detailed)'
        ),
    ]
    manifest: Annotated[
        dict[str, Any],
        Field(
            description='Asset manifest for rendering the detailed card, structure defined by the format'
        ),
    ]


class MetricOptimization(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    supported_metrics: Annotated[
        list[SupportedMetric],
        Field(
            description='Metric kinds this product can optimize for. Buyers should only request metric goals for kinds listed here.',
            min_length=1,
        ),
    ]
    supported_reach_units: Annotated[
        list[reach_unit.ReachUnit] | None,
        Field(
            description="Reach units this product can optimize for. Required when supported_metrics includes 'reach'. Buyers must set reach_unit to a value in this list on reach optimization goals — sellers reject unsupported values.",
            min_length=1,
        ),
    ] = None
    supported_targets: Annotated[
        list[SupportedTarget1] | None,
        Field(
            description='Target kinds available for metric goals on this product. Values match target.kind on the optimization goal. Only these target kinds are accepted — goals with unlisted target kinds will be rejected. When omitted, buyers can set target-less metric goals (maximize volume within budget) but cannot set specific targets.'
        ),
    ] = None
    supported_view_durations: Annotated[
        list[SupportedViewDuration] | None,
        Field(
            description="Video view duration thresholds (in seconds) this product supports for completed_views goals. Only relevant when supported_metrics includes 'completed_views'. When absent, the seller uses their platform default. Buyers must set view_duration_seconds to a value in this list — sellers reject unsupported values."
        ),
    ] = None


class Product(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    brief_relevance: Annotated[
        str | None,
        Field(
            description='Explanation of why this product matches the brief (only included when brief is provided)'
        ),
    ] = None
    catalog_match: Annotated[
        CatalogMatch | None,
        Field(
            description='When the buyer provides a catalog on get_products, indicates which catalog items are eligible for this product. Only present for products where catalog matching is relevant (e.g., sponsored product listings, job boards, hotel ads).'
        ),
    ] = None
    catalog_types: Annotated[
        list[catalog_type.CatalogType] | None,
        Field(
            description='Catalog types this product supports for catalog-driven campaigns. A sponsored product listing declares ["product"], a job board declares ["job", "offering"]. Buyers match synced catalogs to products via this field.',
            min_length=1,
        ),
    ] = None
    channels: Annotated[
        list[channels_1.MediaChannel] | None,
        Field(
            description="Advertising channels this product is sold as. Products inherit from their properties' supported_channels but may narrow the scope. For example, a product covering YouTube properties might be sold as ['ctv'] even though those properties support ['olv', 'social', 'ctv']."
        ),
    ] = None
    conversion_tracking: Annotated[
        ConversionTracking | None,
        Field(
            description="Conversion event tracking for this product. Presence indicates the product supports optimization_goals with kind: 'event'. Seller-level capabilities (supported event types, UID types, attribution windows) are declared in get_adcp_capabilities."
        ),
    ] = None
    creative_policy: creative_policy_1.CreativePolicy | None = None
    data_provider_signals: Annotated[
        list[data_provider_signal_selector.DataProviderSignalSelector] | None,
        Field(
            description="Data provider signals available for this product. Buyers fetch signal definitions from each data provider's adagents.json and can verify agent authorization."
        ),
    ] = None
    delivery_measurement: Annotated[
        DeliveryMeasurement,
        Field(
            description='Measurement provider and methodology for delivery metrics. The buyer accepts the declared provider as the source of truth for the buy. REQUIRED for all products.'
        ),
    ]
    delivery_type: delivery_type_1.DeliveryType
    description: Annotated[
        str, Field(description='Detailed description of the product and its inventory')
    ]
    expires_at: Annotated[
        AwareDatetime | None, Field(description='Expiration timestamp for custom products')
    ] = None
    ext: ext_1.ExtensionObject | None = None
    forecast: Annotated[
        delivery_forecast.DeliveryForecast | None,
        Field(
            description='Forecasted delivery metrics for this product. Gives buyers an estimate of expected performance before requesting a proposal.'
        ),
    ] = None
    format_ids: Annotated[
        list[format_id_1.FormatId],
        Field(
            description='Array of supported creative format IDs - structured format_id objects with agent_url and id'
        ),
    ]
    is_custom: Annotated[bool | None, Field(description='Whether this is a custom product')] = None
    max_optimization_goals: Annotated[
        int | None,
        Field(
            description='Maximum number of optimization_goals this product accepts on a package. When absent, no limit is declared. Most social platforms accept only 1 goal — buyers sending arrays longer than this value should expect the seller to use only the highest-priority (lowest priority number) goal.',
            ge=1,
        ),
    ] = None
    metric_optimization: Annotated[
        MetricOptimization | None,
        Field(
            description="Metric optimization capabilities for this product. Presence indicates the product supports optimization_goals with kind: 'metric'. No event source or conversion tracking setup required — the seller tracks these metrics natively."
        ),
    ] = None
    name: Annotated[str, Field(description='Human-readable product name')]
    outcome_measurement: outcome_measurement_1.OutcomeMeasurement | None = None
    placements: Annotated[
        list[placement.Placement] | None,
        Field(
            description='Optional array of specific placements within this product. When provided, buyers can target specific placements when assigning creatives.',
            min_length=1,
        ),
    ] = None
    pricing_options: Annotated[
        list[pricing_option.PricingOption],
        Field(description='Available pricing models for this product', min_length=1),
    ]
    product_card: Annotated[
        ProductCard | None,
        Field(
            description='Optional standard visual card (300x400px) for displaying this product in user interfaces. Can be rendered via preview_creative or pre-generated.'
        ),
    ] = None
    product_card_detailed: Annotated[
        ProductCardDetailed | None,
        Field(
            description='Optional detailed card with carousel and full specifications. Provides rich product presentation similar to media kit pages.'
        ),
    ] = None
    product_id: Annotated[str, Field(description='Unique identifier for the product')]
    property_targeting_allowed: Annotated[
        bool | None,
        Field(
            description="Whether buyers can filter this product to a subset of its publisher_properties. When false (default), the product is 'all or nothing' - buyers must accept all properties or the product is excluded from property_list filtering results."
        ),
    ] = False
    publisher_properties: Annotated[
        list[publisher_property_selector.PublisherPropertySelector],
        Field(
            description="Publisher properties covered by this product. Buyers fetch actual property definitions from each publisher's adagents.json and validate agent authorization. Selection patterns mirror the authorization patterns in adagents.json for consistency.",
            min_length=1,
        ),
    ]
    reporting_capabilities: reporting_capabilities_1.ReportingCapabilities | None = None
    signal_targeting_allowed: Annotated[
        bool | None,
        Field(
            description='Whether buyers can filter this product to a subset of its data_provider_signals. When false (default), the product includes all listed signals as a bundle. When true, buyers can target specific signals.'
        ),
    ] = False



# generated by datamodel-codegen:
#   filename:  enums/disclosure_position.json
#   timestamp: 2026-02-28T17:39:50+00:00

from __future__ import annotations

from enum import Enum


class DisclosurePosition(Enum):
    prominent = 'prominent'
    footer = 'footer'
    audio = 'audio'
    subtitle = 'subtitle'
    overlay = 'overlay'
    end_card = 'end_card'
    pre_roll = 'pre_roll'
    companion = 'companion'



# generated by datamodel-codegen:
#   filename:  core/requirements/css_asset_requirements.json
#   timestamp: 2026-02-23T01:56:40+00:00

from __future__ import annotations

from typing import Annotated

from adcp.types.base import AdCPBaseModel
from pydantic import ConfigDict, Field


class CssAssetRequirements(AdCPBaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    max_file_size_kb: Annotated[
        int | None, Field(description='Maximum file size in kilobytes', ge=1)
    ] = None
